from cookiecutter.main import cookiecutter


def project_name(safe_repo_name: str) -> str:
    return " ".join(word.capitalize() for word in safe_repo_name.split("-"))


def package_name(safe_repo_name: str) -> str:
    return safe_repo_name.replace("-", "_")


class Context:
    def __init__(  # noqa: WPS211 too-many-arguments
        self,
        name: str,
        email: str,
        url: str,
        full_name: str,
        src_dir: str,
    ) -> None:
        self._name = name
        self._email = email
        self._git_url = url
        self._docker_repo = full_name
        self.repo_name = full_name.split("/")[-1]
        self._project_name = project_name(safe_repo_name=self.repo_name)
        self._package_name = package_name(safe_repo_name=self.repo_name)
        self._src_dir = src_dir

    def dict(self) -> dict[str, str]:
        return {
            "author_email": self._email,
            "author_name": self._name,
            "git_url": self._git_url,
            "package_name": self._package_name,
            "project_name": self._project_name,
            "repo_name": self.repo_name,
            "docker_repo": self._docker_repo,
            "src_dir": self._src_dir,
        }


def generate_template(template_url: str, context: Context) -> None:
    cookiecutter(
        template=template_url,
        extra_context=context.dict(),
        no_input=True,
    )



import os
import subprocess

from github.Repository import Repository


def _git(*command: str) -> None:
    subprocess.run(["git", *command])


def init_template(repository: Repository) -> None:
    os.chdir(repository.name)
    _git("init")
    _git("add", ".")
    _git("commit", "-m", ":tada: Initialize template from nymann/python-template")
    _git("remote", "add", "origin", repository.ssh_url)
    _git("push", "-u", "origin", "master")



from pathlib import Path

from github import Github
from github.Repository import Repository
import typer

from racoon.argument_types import DefaultSrcDir
from racoon.argument_types import DefaultTemplateURL
from racoon.argument_types import RequiredAccessToken
from racoon.git import init_template
from racoon.github_integration import GitHubIntegration
from racoon.template_generation import Context
from racoon.template_generation import generate_template

app = typer.Typer()


def read_file(path: Path) -> str:
    with open(path, "r") as file_pointer:
        return file_pointer.read().strip("\n")


def get_full_name_from_url(url: str) -> str:
    interesting: str = url[url.index(".") :]
    parts: list[str] = interesting.split("/")[1:]
    return "/".join(parts)


def get_repo_by_url(url: str, github: Github) -> Repository:
    full_name = get_full_name_from_url(url=url)
    return github.get_repo(full_name_or_id=full_name)


@app.command()
def generate(
    url: str,
    access_token: Path = RequiredAccessToken,
    src_dir: str = DefaultSrcDir,
    template_url: str = DefaultTemplateURL,
) -> None:
    github = Github(login_or_token=read_file(path=access_token))
    repo: Repository = get_repo_by_url(github=github, url=url)
    user = github.get_user()
    context = Context(
        name=user.name,
        email=user.email,
        url=url,
        src_dir=src_dir,
        full_name=get_full_name_from_url(url=url),
    )
    generate_template(template_url=template_url, context=context)
    github_integration = GitHubIntegration(github=github)
    repo = github_integration.setup(repo=repo)
    init_template(repository=repo)


if __name__ == "__main__":
    app()



from pathlib import Path

import typer

DEFAULT_ACCESS_TOKEN = Path.home().joinpath(".cache/github_token")
RequiredAccessToken: Path = typer.Option(default=DEFAULT_ACCESS_TOKEN, envvar="GITHUB_ACCESS_TOKEN")
DefaultSrcDir: str = typer.Option("src")
DefaultTemplateURL: str = typer.Option("https://github.com/nymann/python-template.git")
